"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AiEditorContribution = void 0;
const tslib_1 = require("tslib");
const debounce_1 = tslib_1.__importDefault(require("lodash/debounce"));
const di_1 = require("@opensumi/di");
const ide_core_browser_1 = require("@opensumi/ide-core-browser");
const browser_1 = require("@opensumi/ide-core-browser/lib/menu/next/renderer/ctxmenu/browser");
const ide_core_common_1 = require("@opensumi/ide-core-common");
const reporter_1 = require("@opensumi/ide-core-common/lib/ai-native/reporter");
const monaco = tslib_1.__importStar(require("@opensumi/ide-monaco"));
const monaco_api_1 = require("@opensumi/ide-monaco/lib/browser/monaco-api");
const telemetry_service_1 = require("@opensumi/ide-monaco/lib/browser/telemetry.service");
const common_1 = require("../common");
const ai_chat_service_1 = require("./ai-chat.service");
const ai_diff_widget_1 = require("./diff-widget/ai-diff-widget");
const inline_chat_feature_registry_1 = require("./inline-chat-widget/inline-chat.feature.registry");
const inline_chat_service_1 = require("./inline-chat-widget/inline-chat.service");
const inline_content_widget_1 = require("./inline-chat-widget/inline-content-widget");
const completeProvider_1 = require("./inline-completions/completeProvider");
const ai_completions_service_1 = require("./inline-completions/service/ai-completions.service");
const parser_1 = require("./languages/parser");
const ai_menu_service_1 = require("./override/ai-menu.service");
const rename_service_1 = require("./rename/rename.service");
const types_1 = require("./types");
let AiEditorContribution = class AiEditorContribution extends ide_core_common_1.Disposable {
    constructor() {
        super();
        /**
         * widget 固定展示
         */
        this.widgetPin = false;
        this.aiInlineChatDisposed = new ide_core_common_1.Disposable();
        this.aiInlineChatOperationDisposed = new ide_core_common_1.Disposable();
        this.inlineChatInUsing = false;
        this.modelRequestSession = new Map();
        this.logger = this.loggerManagerClient.getLogger(ide_core_common_1.SupportLogNamespace.Browser);
    }
    disposeAllWidget(unpin) {
        if (unpin) {
            this.widgetPin = false;
        }
        if (this.widgetPin) {
            return;
        }
        if (this.aiDiffWidget) {
            this.aiDiffWidget.dispose();
        }
        if (this.aiInlineContentWidget) {
            this.aiInlineContentWidget.dispose();
        }
        if (this.aiInlineChatDisposed) {
            this.aiInlineChatDisposed.dispose();
        }
        if (this.aiInlineChatOperationDisposed) {
            this.aiInlineChatOperationDisposed.dispose();
        }
        this.aiChatService.cancelToken();
        this.inlineChatInUsing = false;
    }
    contribute(editor) {
        if (!editor) {
            return this;
        }
        const { monacoEditor, currentUri } = editor;
        if (currentUri && currentUri.codeUri.scheme !== ide_core_common_1.Schemes.file) {
            return this;
        }
        if (this.aiNativeConfigService.capabilities.supportsInlineCompletion) {
            this.contributions.getContributions().forEach((contribution) => {
                if (contribution.middleware) {
                    this.latestMiddlewareCollector = contribution.middleware;
                }
            });
            this.registerCompletion(editor);
        }
        if (this.aiNativeConfigService.capabilities.supportsInlineChat) {
            this.registerCodeActionForInlineChat(editor);
        }
        if (this.aiNativeConfigService.capabilities.supportsRenameSuggestions) {
            this.registerRenameSuggestions(editor);
        }
        this.disposables.push(monacoEditor.onWillChangeModel(() => {
            this.disposeAllWidget(true);
        }));
        this.disposables.push(this.inlineChatFeatureRegistry.onActionRun(({ id, range }) => {
            monacoEditor.setSelection(range);
            this.registerInlineChat(editor, id);
        }));
        this.disposables.push(monacoEditor.onDidScrollChange(() => {
            /**
             * 其他的 ctxmenu 服务注册的菜单在 onHide 函数里会有其他逻辑处理，例如在 editor.context.ts 会在 hide 的时候 focus 编辑器，影响使用
             */
            this.ctxMenuRenderer.onHide = undefined;
            this.ctxMenuRenderer.hide(true);
        }));
        this.disposables.push(this.aiChatService.onInlineChatVisible((value) => {
            if (value) {
                this.registerInlineChat(editor);
            }
            else {
                this.disposeAllWidget(true);
            }
        }));
        let isShowInlineChat = false;
        this.disposables.push(monacoEditor.onMouseDown(() => {
            isShowInlineChat = false;
        }), monacoEditor.onMouseUp((event) => {
            const target = event.target;
            const detail = target.detail;
            if (detail && typeof detail === 'string' && detail === common_1.AiInlineChatContentWidget) {
                isShowInlineChat = false;
            }
            else {
                isShowInlineChat = true;
            }
        }));
        let prefInlineChatAutoVisible = this.preferenceService.getValid(common_1.AiNativeSettingSectionsId.INLINE_CHAT_AUTO_VISIBLE, true);
        this.disposables.push({
            dispose: () => {
                this.preferenceService.onSpecificPreferenceChange(common_1.AiNativeSettingSectionsId.INLINE_CHAT_AUTO_VISIBLE, ({ newValue }) => {
                    prefInlineChatAutoVisible = newValue;
                });
            },
        });
        this.debounceInlineChatHandler = (0, debounce_1.default)(() => {
            if (this.widgetPin) {
                return;
            }
            this.disposeAllWidget();
            if (!prefInlineChatAutoVisible) {
                return;
            }
            if (!isShowInlineChat) {
                return;
            }
            if (this.aiInlineChatService.status !== 0 /* EInlineChatStatus.READY */ &&
                this.aiInlineChatService.status !== 3 /* EInlineChatStatus.ERROR */) {
                return;
            }
            this.registerInlineChat(editor);
        });
        ide_core_common_1.Event.any(monacoEditor.onDidChangeCursorSelection, monacoEditor.onMouseUp)(this.debounceInlineChatHandler);
        this.logger.log('AiEditorContribution:>>>', editor, monacoEditor);
        return this;
    }
    /**
     * 新版 inline chat（类似 cursor 那种）
     */
    async registerInlineChat(editor, runId) {
        if (this.aiNativeConfigService.capabilities.supportsInlineChat === false) {
            return;
        }
        if (this.inlineChatInUsing) {
            return;
        }
        this.inlineChatInUsing = true;
        this.disposeAllWidget();
        // 取消所有的在途的 debounce 函数
        if (this.debounceInlineChatHandler) {
            this.debounceInlineChatHandler.cancel();
        }
        const { monacoEditor, currentUri } = editor;
        const runByCodeAction = Boolean(runId);
        if (!currentUri) {
            return;
        }
        if (currentUri && currentUri.codeUri.scheme !== ide_core_common_1.Schemes.file) {
            return;
        }
        const selection = monacoEditor.getSelection();
        if (!selection) {
            return;
        }
        if (selection.isEmpty()) {
            return;
        }
        this.widgetPin = true;
        this.aiInlineChatService.launchChatStatus(0 /* EInlineChatStatus.READY */);
        this.aiInlineContentWidget = this.injector.get(inline_content_widget_1.AiInlineContentWidget, [monacoEditor]);
        this.aiInlineContentWidget.show({
            selection,
        });
        this.aiInlineChatDisposed.addDispose(this.aiInlineContentWidget.onClickActions(async (id) => {
            const handler = this.inlineChatFeatureRegistry.getHandler(id);
            const action = this.inlineChatFeatureRegistry.getAction(id);
            if (!handler || !action) {
                this.disposeAllWidget(true);
                return;
            }
            const { execute, providerDiffPreviewStrategy } = handler;
            if (execute) {
                this.aiInlineChatService.launchChatStatus(1 /* EInlineChatStatus.THINKING */);
                await execute(editor);
                this.disposeAllWidget(true);
            }
            if (providerDiffPreviewStrategy) {
                const crossSelection = selection
                    .setStartPosition(selection.startLineNumber, 1)
                    .setEndPosition(selection.endLineNumber, Number.MAX_SAFE_INTEGER);
                const relationId = this.aiReporter.start(action.name, { message: action.name, runByCodeAction });
                const startTime = +new Date();
                const result = await this.handleDiffPreviewStrategy(editor, providerDiffPreviewStrategy, crossSelection, relationId);
                this.aiReporter.end(relationId, { message: result, success: !!result, replytime: +new Date() - startTime });
                this.aiInlineChatDisposed.addDispose(this.aiInlineChatService.onDiscard(() => {
                    this.aiReporter.end(relationId, { message: result, success: !!result, isDrop: true });
                    this.disposeAllWidget(true);
                }));
                this.aiInlineChatDisposed.addDispose(this.aiInlineChatService.onRegenerate(async () => {
                    const retryStartTime = +new Date();
                    const retryResult = await this.handleDiffPreviewStrategy(editor, providerDiffPreviewStrategy, crossSelection, relationId);
                    this.aiReporter.end(relationId, {
                        message: retryResult,
                        success: !!result,
                        replytime: +new Date() - retryStartTime,
                        isRetry: true,
                    });
                }));
            }
        }));
        if (runId) {
            this.aiInlineContentWidget.clickActions(runId);
        }
    }
    async handleDiffPreviewStrategy(editor, strategy, crossSelection, relationId) {
        var _a, _b;
        const model = editor.monacoEditor.getModel();
        if (!model) {
            return;
        }
        if (this.aiDiffWidget) {
            this.aiDiffWidget.dispose();
        }
        if (this.aiInlineChatOperationDisposed) {
            this.aiInlineChatOperationDisposed.dispose();
        }
        const crossCode = model.getValueInRange(crossSelection);
        this.aiInlineChatService.launchChatStatus(1 /* EInlineChatStatus.THINKING */);
        const response = await strategy(editor, this.aiChatService.cancelIndicator.token);
        if (this.aiInlineChatDisposed.disposed || types_1.CancelResponse.is(response)) {
            this.aiInlineChatService.launchChatStatus(0 /* EInlineChatStatus.READY */);
            return;
        }
        if (types_1.ErrorResponse.is(response)) {
            this.aiInlineChatService.launchChatStatus(3 /* EInlineChatStatus.ERROR */);
            return;
        }
        else {
            this.aiInlineChatService.launchChatStatus(2 /* EInlineChatStatus.DONE */);
        }
        let answer = response && response.message;
        // 提取代码内容
        const regex = /```\w*([\s\S]+?)\s*```/;
        const regExec = regex.exec(answer);
        answer = (regExec && regExec[1]) || answer;
        this.logger.log('fetch response answer:>>>> ', answer);
        if (answer) {
            const regex = /^\s*/;
            const matches = crossCode.match(regex);
            let spaceCount = 4;
            if (matches) {
                spaceCount = matches[0].length;
            }
            const indents = ' '.repeat(spaceCount);
            const spcode = answer.split('\n');
            answer = spcode.map((s, i) => (i === 0 ? s : indents + s)).join('\n');
            editor.monacoEditor.setHiddenAreas([crossSelection], ai_diff_widget_1.AiDiffWidget._hideId);
            this.aiDiffWidget = this.injector.get(ai_diff_widget_1.AiDiffWidget, [editor.monacoEditor, crossSelection, answer]);
            this.aiDiffWidget.create();
            this.aiDiffWidget.showByLine(crossSelection.startLineNumber - 1, crossSelection.endLineNumber - crossSelection.startLineNumber + 2);
            (_a = this.aiInlineContentWidget) === null || _a === void 0 ? void 0 : _a.setOptions({
                position: {
                    lineNumber: crossSelection.endLineNumber + 1,
                    column: 1,
                },
            });
            (_b = this.aiInlineContentWidget) === null || _b === void 0 ? void 0 : _b.layoutContentWidget();
            this.aiInlineChatOperationDisposed.addDispose([
                this.aiInlineChatService.onAccept(() => {
                    var _a;
                    this.aiReporter.end(relationId, { message: 'accept', success: true, isReceive: true });
                    (_a = editor.monacoEditor.getModel()) === null || _a === void 0 ? void 0 : _a.pushEditOperations(null, [
                        {
                            range: crossSelection,
                            text: answer,
                        },
                    ], () => null);
                    setTimeout(() => {
                        this.disposeAllWidget(true);
                    }, 110);
                }),
                this.aiInlineChatService.onThumbs((isLike) => {
                    this.aiReporter.end(relationId, { isLike });
                }),
                this.aiDiffWidget.onMaxLincCount((count) => {
                    requestAnimationFrame(() => {
                        if (crossSelection.endLineNumber === model.getLineCount()) {
                            const lineHeight = editor.monacoEditor.getOption(monaco.editor.EditorOption.lineHeight);
                            this.aiInlineContentWidget.offsetTop(lineHeight * count + 12);
                        }
                    });
                }),
            ]);
        }
        return answer;
    }
    provideEditorOptionsForUri(uri) {
        throw new Error('Method not implemented.');
    }
    /**
     * 代码补全
     */
    async registerCompletion(editor) {
        const { monacoEditor, currentUri } = editor;
        if (currentUri && currentUri.codeUri.scheme !== ide_core_common_1.Schemes.file) {
            return;
        }
        let dispose;
        this.disposables.push(ide_core_common_1.Event.debounce(monacoEditor.onWillChangeModel, (_, e) => e, 300)(async (event) => {
            if (dispose) {
                dispose.dispose();
                this.aiInlineCompletionsProvider.dispose();
            }
            const { monacoEditor, currentUri } = editor;
            if (currentUri && currentUri.codeUri.scheme !== ide_core_common_1.Schemes.file) {
                return this;
            }
            const model = monacoEditor.getModel();
            if (!model) {
                return;
            }
            // 取光标的当前位置
            const position = monacoEditor.getPosition();
            if (!position) {
                return;
            }
            this.aiInlineCompletionsProvider.registerEditor(editor);
            dispose = monaco_api_1.monaco.languages.registerInlineCompletionsProvider(model.getLanguageId(), {
                provideInlineCompletions: async (model, position, context, token) => {
                    var _a, _b, _c, _d;
                    if ((_b = (_a = this.latestMiddlewareCollector) === null || _a === void 0 ? void 0 : _a.language) === null || _b === void 0 ? void 0 : _b.provideInlineCompletions) {
                        this.aiCompletionsService.setMiddlewareComplete((_d = (_c = this.latestMiddlewareCollector) === null || _c === void 0 ? void 0 : _c.language) === null || _d === void 0 ? void 0 : _d.provideInlineCompletions);
                    }
                    const list = await this.aiInlineCompletionsProvider.provideInlineCompletionItems(model, position, context, token);
                    this.logger.log('provideInlineCompletions:>>>> ', list.items.map((data) => data.insertText));
                    return list;
                },
                freeInlineCompletions(completions) { },
                handleItemDidShow: (completions, item) => {
                    if (completions.items.length > 0) {
                        this.aiCompletionsService.setVisibleCompletion(true);
                    }
                },
            });
            this.disposables.push(dispose);
        }));
        return;
    }
    /**
     * 代码补全
     */
    registerCodeActionForInlineChat(editor) {
        const { monacoEditor, currentUri } = editor;
        if (currentUri && currentUri.codeUri.scheme !== ide_core_common_1.Schemes.file) {
            return;
        }
        let dispose;
        this.disposables.push(ide_core_common_1.Event.debounce(monacoEditor.onWillChangeModel, (_, e) => e, 300)(async (event) => {
            if (dispose) {
                dispose.dispose();
            }
            const { monacoEditor, currentUri } = editor;
            if (currentUri && currentUri.codeUri.scheme !== ide_core_common_1.Schemes.file) {
                return this;
            }
            const model = monacoEditor.getModel();
            if (!model) {
                return;
            }
            // 取光标的当前位置
            const position = monacoEditor.getPosition();
            if (!position) {
                return;
            }
            const languageId = model.getLanguageId();
            dispose = monaco_api_1.monaco.languages.registerCodeActionProvider(languageId, {
                provideCodeActions: async (model, position, context, token) => {
                    const parser = this.languageParserFactory(languageId);
                    if (!parser) {
                        return null;
                    }
                    const cursorPosition = monacoEditor.getPosition();
                    const actions = this.inlineChatFeatureRegistry.getCodeActions();
                    const functionInfo = await parser.provideFunctionInfo(model, cursorPosition);
                    if (functionInfo) {
                        return {
                            actions: actions.map((v) => {
                                const command = {};
                                if (v.command) {
                                    command.id = v.command.id;
                                    command.arguments = [functionInfo.range];
                                }
                                return Object.assign(Object.assign({}, v), { title: v.title + ` for Function: ${functionInfo.name}`, ranges: [functionInfo.range], command });
                            }),
                            dispose() { },
                        };
                    }
                    const codeblock = await parser.findCodeBlock(model, cursorPosition);
                    if (codeblock) {
                        return {
                            actions: actions.map((v) => {
                                const command = {};
                                if (v.command) {
                                    command.id = v.command.id;
                                    command.arguments = [codeblock.range];
                                }
                                return Object.assign(Object.assign({}, v), { title: v.title, ranges: [codeblock.range], command });
                            }),
                            dispose() { },
                        };
                    }
                },
            });
            this.disposables.push(dispose);
        }));
        return;
    }
    async registerRenameSuggestions(editor) {
        const { monacoEditor, currentUri } = editor;
        if (currentUri && currentUri.codeUri.scheme !== ide_core_common_1.Schemes.file) {
            return;
        }
        let dispose = new ide_core_common_1.Disposable();
        this.disposables.push(ide_core_common_1.Event.debounce(monacoEditor.onWillChangeModel, (_, e) => e, 300)(async (event) => {
            if (dispose) {
                dispose.dispose();
            }
            dispose = new ide_core_common_1.Disposable();
            const model = monacoEditor.getModel();
            if (!model) {
                return;
            }
            dispose.addDispose(this.monacoTelemetryService.onEventLog(({ type, event }) => {
                if (type === 'renameInvokedEvent' && this.lastmodelRequestRenameSessionId) {
                    this.aiReporter.end(this.lastmodelRequestRenameSessionId, Object.assign({ message: 'rename done', success: true, modelRequestEndTime: this.lastmodelRequestRenameEndTime }, event));
                }
            }));
            const provider = async (model, range, token) => {
                this.lastmodelRequestRenameSessionId = undefined;
                const startTime = +new Date();
                const relationId = this.aiReporter.start('rename', {
                    message: 'start',
                    type: reporter_1.AISerivceType.Rename,
                    modelRequestStartTime: startTime,
                });
                this.lastmodelRequestRenameSessionId = relationId;
                const dispose1 = token.onCancellationRequested(() => {
                    const endTime = +new Date();
                    this.aiReporter.end(relationId, {
                        message: 'cancel',
                        success: false,
                        isCancel: true,
                        modelRequestStartTime: startTime,
                        modelRequestEndTime: endTime,
                    });
                    this.lastmodelRequestRenameSessionId = undefined;
                });
                try {
                    const result = await this.renameSuggestionService.provideRenameSuggestions(model, range, token);
                    dispose1.dispose();
                    this.lastmodelRequestRenameEndTime = +new Date();
                    return result;
                }
                catch (error) {
                    const endTime = +new Date();
                    this.aiReporter.end(relationId, {
                        message: 'error:' + (0, ide_core_common_1.getErrorMessage)(error),
                        success: false,
                        modelRequestStartTime: startTime,
                        modelRequestEndTime: endTime,
                    });
                    throw error;
                }
            };
            dispose.addDispose(monaco_api_1.monaco.languages.registerNewSymbolNameProvider(model.getLanguageId(), {
                provideNewSymbolNames: provider,
            }));
        }));
    }
};
tslib_1.__decorate([
    (0, di_1.Autowired)(di_1.INJECTOR_TOKEN),
    tslib_1.__metadata("design:type", di_1.Injector)
], AiEditorContribution.prototype, "injector", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.AiNativeConfigService),
    tslib_1.__metadata("design:type", ide_core_browser_1.AiNativeConfigService)
], AiEditorContribution.prototype, "aiNativeConfigService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.IAiInlineChatService),
    tslib_1.__metadata("design:type", inline_chat_service_1.AiInlineChatService)
], AiEditorContribution.prototype, "aiInlineChatService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ai_chat_service_1.AiChatService),
    tslib_1.__metadata("design:type", ai_chat_service_1.AiChatService)
], AiEditorContribution.prototype, "aiChatService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_common_1.ILoggerManagerClient),
    tslib_1.__metadata("design:type", Object)
], AiEditorContribution.prototype, "loggerManagerClient", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ide_core_browser_1.PreferenceService),
    tslib_1.__metadata("design:type", Object)
], AiEditorContribution.prototype, "preferenceService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(browser_1.IBrowserCtxMenu),
    tslib_1.__metadata("design:type", ai_menu_service_1.AiBrowserCtxMenuService)
], AiEditorContribution.prototype, "ctxMenuRenderer", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(completeProvider_1.AiInlineCompletionsProvider),
    tslib_1.__metadata("design:type", completeProvider_1.AiInlineCompletionsProvider)
], AiEditorContribution.prototype, "aiInlineCompletionsProvider", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(types_1.IInlineChatFeatureRegistry),
    tslib_1.__metadata("design:type", inline_chat_feature_registry_1.InlineChatFeatureRegistry)
], AiEditorContribution.prototype, "inlineChatFeatureRegistry", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(reporter_1.IAIReporter),
    tslib_1.__metadata("design:type", Object)
], AiEditorContribution.prototype, "aiReporter", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(types_1.AiNativeCoreContribution),
    tslib_1.__metadata("design:type", Object)
], AiEditorContribution.prototype, "contributions", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(ai_completions_service_1.AiCompletionsService),
    tslib_1.__metadata("design:type", ai_completions_service_1.AiCompletionsService)
], AiEditorContribution.prototype, "aiCompletionsService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(rename_service_1.RenameSuggestionsService),
    tslib_1.__metadata("design:type", rename_service_1.RenameSuggestionsService)
], AiEditorContribution.prototype, "renameSuggestionService", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(parser_1.LanguageParserFactory),
    tslib_1.__metadata("design:type", Function)
], AiEditorContribution.prototype, "languageParserFactory", void 0);
tslib_1.__decorate([
    (0, di_1.Autowired)(),
    tslib_1.__metadata("design:type", telemetry_service_1.MonacoTelemetryService)
], AiEditorContribution.prototype, "monacoTelemetryService", void 0);
AiEditorContribution = tslib_1.__decorate([
    (0, di_1.Injectable)(),
    tslib_1.__metadata("design:paramtypes", [])
], AiEditorContribution);
exports.AiEditorContribution = AiEditorContribution;
//# sourceMappingURL=ai-editor.contribution.js.map